/************************************************************************/
/*                                                                      */
/*                      Debouncing 8 Keys				*/
/*			Sampling 4 Times				*/
/*                                                                      */
/*              Author: Peter Dannegger                                 */
/*                                                                      */
/************************************************************************/
#include "main.h"
#include "getkey.h"


#define REPEAT_MASK	(1<<KEY0)		// repeat: key0
#define REPEAT_START	50			// after 500ms
#define REPEAT_NEXT	20			// every 200ms


time_t  time_10ms;				// time base 10ms

uint8_t key_state;				// debounced and inverted key state:
						// bit = 1: key pressed
uint8_t key_press;				// key press detect

uint8_t key_rpt;				// key long press and repeat


ISR( TIMER0_COMPA_vect )			// every 10ms
{
  static uint8_t ct0 = 0xFF, ct1 = 0xFF;	// 8 * 2bit counters
  static uint8_t rpt;
  uint8_t i;

  i = key_state ^ KEY_PIN;			// key changed ?
  ct0 = ~( ct0 & i );				// reset or count ct0
  ct1 = ct0 ^ (ct1 & i);			// reset or count ct1
  i &= ct0 & ct1;				// count until roll over ?
  key_state ^= i;				// then toggle debounced state
  key_press |= key_state & i;			// 0->1: key press detect

  if( (key_state & REPEAT_MASK) == 0 )		// check repeat function
     rpt = REPEAT_START;			// start delay
  if( --rpt == 0 ){
    rpt = REPEAT_NEXT;				// repeat delay
    key_rpt |= key_state & REPEAT_MASK;
  }
  time_10ms++;
}


time_t get_time_10ms()
{
  time_t t;

  ATOMIC_BLOCK(ATOMIC_FORCEON){			// read atomic
    t = time_10ms;
  }  
  return t;
}


uint8_t get_key_press( uint8_t key_mask )
{
  ATOMIC_BLOCK(ATOMIC_FORCEON){			// read and clear atomic
    key_mask &= key_press;			// read key(s)
    key_press ^= key_mask;			// clear key(s)
  }
  return key_mask;
}

uint8_t get_key_rpt( uint8_t key_mask )
{
  ATOMIC_BLOCK(ATOMIC_FORCEON){			// read and clear atomic
    key_mask &= key_rpt;			// read key(s)
    key_rpt ^= key_mask;			// clear key(s)
  }
  return key_mask;
}


uint8_t get_key_short( uint8_t key_mask )
{
  cli();					// read key state and key press atomic
  return get_key_press( ~key_state & key_mask );
}


uint8_t get_key_long( uint8_t key_mask )
{
  return get_key_press( get_key_rpt( key_mask ));
}


void get_key_init()
{
  TCCR0A = 1<<WGM01;			// T0 Mode 2: CTC
  TCCR0B = 1<<CS02;			// divide by 256
  OCR0A = F_CPU / 256.0 * 10e-3 - 1;	// 10ms
  TIMSK0 |= 1<<OCIE0A;			// enable T0 interrupt

  key_state = ~KEY_PIN;			// no action on keypress during reset
}
