#include "main.h"
#include "knightrider.h"
#include "getkey.h"


time_t time;


void init( void )
{
  UNUSED = 0xFF;
  KEY_PULLUP = 0xFF;
  LED_PORT = 0xFF;				// all LEDs off
  LED_DDR = 0xFF;
  ACSR = 1<<ACD;				// analog comparator off

  get_key_init();

  PCMSK_REG = 1<<PCINT_BIT;			// prepare awake pin
  sleep_enable();
}


uint8_t get_key_busy( uint8_t key_mask )
{
  return ((KEY_PIN ^ key_state) | key_press) & key_mask;	// until key released
}


void try_sleep()
{
  if( get_key_busy( 1<<KEY0 ))
    return;					// still busy
  LED_PORT = 0xFF;				// all LEDs off
  set_sleep_mode( SLEEP_MODE_PWR_DOWN);		// prepare power dwn
  PCICR = 1<<PCICR_ENABLE;			// awake interrupt on
  sleep_cpu();
  PCICR = 0;
}


EMPTY_INTERRUPT( PCINT_vect );			// wake up from power down


#define TIMEOUT		6000			// 1min



int main()
{
  uint8_t sample = 1;
  uint8_t stop = 0;

  select( 1 );					// select first pattern
  time = get_time_10ms();
  init();
  sei();

  for(;;){
    if( get_key_long( 1<<KEY0 )){		// long press: sleep / awake
      time = get_time_10ms();
      stop ^= 0xFF;
    }
    if( stop ){
      try_sleep();
      LED_PORT = 0x00;				// all LEDs on (for debug only)
      get_key_short( 1<<KEY0 );			// ignore short press during stop
    }else{
      set_sleep_mode( SLEEP_MODE_IDLE);		// prepare idle
      sleep_cpu();				// idle
      if( get_key_short( 1<<KEY0 )){		// short press: select pattern
        time = get_time_10ms();
        sample++;
        while( select( sample ))		// on overflow:
          sample = 1;				// restart with first pattern
      }
      run_pattern();
      if( get_time_10ms() - time > TIMEOUT )
        stop = 0xFF;
    }
  }
}
